from ehelply_bootstrapper.integrations.integration import Integration
from ehelply_bootstrapper.utils.state import State
from datetime import datetime
from typing import Union
from pydantic import BaseModel
from ehelply_microservice_library.integrations.fact import get_fact_endpoint
from ehelply_microservice_library.integrations.sdk import SDK

from ehelply_python_sdk.services.meta.sdk import *


class Meta(Integration):
    """
    Meta integration is used to talk to the ehelply-meta microservice
    """

    def __init__(self) -> None:
        super().__init__("meta")

        class IntegratedSDK(MetaSDK):
            def get_base_url(self) -> str:
                return get_fact_endpoint('ehelply-meta')

        self.sdk: IntegratedSDK = IntegratedSDK(
            sdk_configuration=SDK.instance.sdk_configuration,
            requests_session=SDK.instance.requests_session
        )

        self.m2m = State.integrations.get("m2m")

    def load(self):
        pass

    def slug(self, name: str):
        """
        Turn a string into a slug
        :param name:
        :return:
        """
        return self.m2m.requests.post(self.get_base_url() + "/slug", json={"name": name}).json()



import asyncio
import traceback

from typing import Union, Optional

from pydantic import BaseModel

from fastapi import HTTPException

from ehelply_bootstrapper.utils.state import State

from ehelply_python_sdk.services.access.auth_rules import AuthRule, AuthException
from ehelply_python_sdk.services.access.sdk import AuthModel, is_response_error

from ehelply_microservice_library.integrations.monitor import Monitor
from ehelply_microservice_library.integrations.m2m import M2M


class CloudParticipant(BaseModel):
    active_participant: str
    project_uuid: str
    is_privileged: bool = False
    entity_identifier: Optional[str] = None


async def ehelply_cloud_access(
        auth: AuthModel,
        node: str,
        service_target: str,
        monitor: Monitor,
        ignore_project_enabled=False,
        ignore_spend_limits=False,
        exception_if_unauthorized=True,
        exception_if_spend_maxed=True,
        exception_if_project_not_enabled=True,
        i_am_monitor=False,
) -> Union[bool, CloudParticipant]:
    """

    Args:
        auth:
        node:
        service_target:
        monitor: Send in the monitor integration and project max spend will be checked automatically
        exception_if_unauthorized:
        exception_if_spend_maxed:

    Returns:

    """

    # If key is a known M2M key to this service, we can just return right out of this
    m2m_integration: M2M = State.integrations.get("m2m")
    if m2m_integration.speedy_m2m_auth(access_token=auth.access_token, secret_token=auth.secret_token):
        return CloudParticipant(
            is_privileged=True,
            active_participant=auth.active_participant_uuid,
            project_uuid=auth.project_uuid,
            entity_identifier=auth.entity_identifier
        )

    # API Requests
    admin_auth = asyncio.create_task(AuthRule(
        auth,
        exception_if_unauthorized=False
    ).participant_has_node_on_target(
        node=node,
        target_identifier=service_target,
        partition="ehelply-resources"
    ).verify())

    cloud_auth = asyncio.create_task(AuthRule(
        auth,
        exception_if_unauthorized=False
    ).participant_has_node_on_target(
        node=node,
        target_identifier=auth.project_uuid,
        partition="ehelply-cloud"
    ).customentity_has_node_on_target(
        node=node,
        target_identifier=service_target,
        partition="ehelply-cloud",
        entity_identifier=auth.project_uuid
    ).verify())

    if not i_am_monitor:
        project = asyncio.create_task(
            monitor.get_project(project_uuid=auth.project_uuid)
        )

    try:
        if not i_am_monitor:
            api_results = await asyncio.gather(admin_auth, cloud_auth, project)
        else:
            api_results = await asyncio.gather(admin_auth, cloud_auth)
    except Exception as e:
        traceback.print_exc()
        State.logger.warning(message="cloud_participant async gather failed. In the past this was due to ehelply-access not responding correctly, but this could occur for other unknown reasons as well.")
        raise HTTPException(
            status_code=500,
            detail="Big sad, something has gone terribly wrong - Denied by eHelply."
        )

    admin_auth = api_results[0]
    cloud_auth = api_results[1]

    if not i_am_monitor:
        project = api_results[2]

    # Admin or M2M access
    try:
        # await AuthRule(
        #     auth,
        #     exception_if_unauthorized=True
        # ).participant_has_node_on_target(
        #     node=node,
        #     target_identifier=service_target,
        #     partition="ehelply-resources"
        # ).verify()
        if not admin_auth:
            raise AuthException()

        return CloudParticipant(
            is_privileged=True,
            active_participant=auth.active_participant_uuid,
            project_uuid=auth.project_uuid,
            entity_identifier=auth.entity_identifier
        )
    except (AuthException, HTTPException):
        pass
    except Exception as e:
        traceback.print_exc()
        State.logger.warning(message="cloud_participant admin/m2m access checks caused an unexpected error.")
        raise HTTPException(
            status_code=500,
            detail="Ruh Roh, something has gone terribly wrong - Denied by eHelply."
        )

    spend_maxed: bool = False
    project_not_enabled: bool = False

    # Regular eHelply Cloud access
    try:
        # await AuthRule(
        #     auth,
        #     exception_if_unauthorized=True
        # ).participant_has_node_on_target(
        #     node=node,
        #     target_identifier=auth.project_uuid,
        #     partition="ehelply-cloud"
        # ).customentity_has_node_on_target(
        #     node=node,
        #     target_identifier=service_target,
        #     partition="ehelply-cloud",
        #     entity_identifier=auth.project_uuid
        # ).verify()
        if not cloud_auth:
            raise AuthException()

        if not i_am_monitor:
            # project = await monitor.get_project(project_uuid=auth.project_uuid)
            if not project or is_response_error(project):
                raise AuthException()

            if not ignore_project_enabled and project.status != "enabled":
                project_not_enabled = True
                raise AuthException("Project is not enabled")

            if not ignore_spend_limits and project.is_spend_maxed:
                spend_maxed = True
                raise AuthException("Spend is maxed")

        return CloudParticipant(
            is_privileged=False,
            active_participant=auth.active_participant_uuid,
            project_uuid=auth.project_uuid,
            entity_identifier=auth.entity_identifier
        )
    except (AuthException, HTTPException):
        pass
    except Exception as e:
        traceback.print_exc()
        State.logger.warning(message="cloud_participant regular ehelply cloud access checks caused an unexpected error.")
        raise HTTPException(
            status_code=500,
            detail="Ruh Roh, something has gone terribly wrong - Denied by eHelply."
        )

    if spend_maxed and exception_if_spend_maxed:
        raise HTTPException(
            status_code=400,
            detail="Project spend is maxed - Denied by eHelply. Increase max spend to complete this request."
        )
    elif spend_maxed:
        return False

    if project_not_enabled and exception_if_project_not_enabled:
        raise HTTPException(
            status_code=400,
            detail="Project is not enabled - Denied by eHelply. The project may be awaiting approval or disabled."
        )
    elif project_not_enabled:
        return False

    if exception_if_unauthorized:
        raise HTTPException(status_code=403, detail="Unauthorized - Denied by eHelply")
    else:
        return False

    #
    # return AuthRule(
    #     auth,
    #     AuthRule(auth).participant_has_node_on_target(
    #         node=node,
    #         target_identifier=service_target,
    #         partition="ehelply-resources"
    #     ),
    #     AuthRule(auth).participant_has_node_on_target(
    #         node=node,
    #         target_identifier=auth.project_uuid,
    #         partition="ehelply-cloud"
    #     ).customentity_has_node_on_target(
    #         node=node,
    #         target_identifier=service_target,
    #         partition="ehelply-cloud",
    #         entity_identifier=auth.project_uuid
    #     ),
    #     exception_if_unauthorized=exception_if_unauthorized
    # ).verify()



import typer
from pathlib import Path
from typing import Optional
from ehelply_bootstrapper.utils.releases import ReleasesConfig, ReleaseDetails, Releaser
import os

cli = typer.Typer()


@cli.command()
def create(
        version: str = typer.Option(..., prompt=True, help="The version of the new release. This MUST always be greater than the previous version. Use semantic versioning"),
        name: str = typer.Option(..., prompt=True, help="Name of the release"),
        changelog: str = typer.Option(..., prompt=True, help="Brief, comma separated changelog"),
        dirty: Optional[bool] = typer.Option(False, help="When false, a release cannot be made if a repository is dirty"),
        releases: Optional[Path] = typer.Option(None, help="Path to the directory where releases.json can be found. By default it is the current working directory")
):
    """
    Create a new microservice release
    """

    base_path: Path = releases
    if not base_path:
        base_path = Path(os.getcwd())

    repo_path: Path = base_path  # Path(Path(__file__).resolve().parents[3])  # TODO: Change back to 3 (4 when testing in template)

    releases_path: Path = base_path.joinpath("releases.json")

    config: ReleasesConfig = ReleasesConfig(
        repo_path=repo_path,
        releases_path=releases_path
    )

    releaser: Releaser = Releaser(config, allow_dirty=dirty)
    releaser.make(ReleaseDetails(name=name, version=version, changelog=changelog))


if __name__ == '__main__':
    cli()



from ehelply_bootstrapper.utils.state import State
from ehelply_bootstrapper.utils.cryptography import Encryption
from ehelply_bootstrapper.integrations.integration import Integration
from ehelply_bootstrapper.drivers.fast_api_utils.responses import *

from ehelply_python_sdk.sdk import SDKConfiguration, eHelplySDK, ErrorResponse, is_response_error
from ehelply_python_sdk.services.service_schemas import GenericHTTPResponse


class SDK(Integration):
    instance: eHelplySDK

    def __init__(self, service_gatekeeper_key: str) -> None:
        super().__init__("sdk")

        self.enc: Encryption = Encryption([service_gatekeeper_key.encode(Encryption.STRING_ENCODING)])

    def init(self):
        try:
            secret_token: str = State.config.m2m.auth.secret_key
            access_token: str = State.config.m2m.auth.access_key

            if len(secret_token) == 0 or len(access_token) == 0:
                State.logger.warning("SDK (M2M) credentials are not set. Check the m2m.yaml config")

            secret_token = self.enc.decrypt_str(secret_token.encode(Encryption.STRING_ENCODING))
            access_token = self.enc.decrypt_str(access_token.encode(Encryption.STRING_ENCODING))

            # Setup SDK
            SDK.instance = SDK.make_sdk(access_token, secret_token)

        except:
            SDK.instance = SDK.make_sdk("", "")
            State.logger.severe(
                "SDK (M2M) credentials are invalid. Ensure they are encrypted. Check the m2m.yaml config")

    @staticmethod
    def make_sdk(access_token: str, secret_token: str) -> eHelplySDK:
        ehelply_sdk: eHelplySDK = eHelplySDK(
            sdk_configuration=SDKConfiguration(
                access_token=access_token,
                secret_token=secret_token,
                project_identifier="ehelply-resources",
                partition_identifier="ehelply-cloud"
            )
        )
        return ehelply_sdk

    @staticmethod
    def make_json_response(response: GenericHTTPResponse) -> JSONResponse:
        if is_response_error(response):
            response: ErrorResponse
            return JSONResponse(status_code=response.status_code, content={"message": response.message})
        content = response.dict()
        content.pop('status_code')
        return JSONResponse(status_code=response.status_code, content=content)
